/* main.c 
   
Partly based on code from xvt 2.1, which is:
Copyright 1992-94, 1997 John Bovey, University of Kent at Canterbury.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <pty.h>
#include "compat.h"
#include "console.h"
#include "x11.h"

int main(int argc, char **argv) {
  char *command = argv[1];
  int ptymaster, ptyslave, pid;
  struct winsize wsize;
  struct termios term;

  printf("initialising\n");
  initialise_console(&x11_con, &wsize);

#warning should set size here - pass a winsize to initialise_console
  if (openpty(&ptymaster, &ptyslave, NULL, NULL, NULL) < 0) {
    exit(20);
  }
  fcntl(ptymaster, F_SETFL, O_NDELAY);
  ioctl(ptyslave, TIOCSWINSZ, (char *)&wsize);


  memset((char *)&term, 0, sizeof(term));
  term.c_iflag = BRKINT | IGNPAR | ICRNL | IXON;
#ifdef IMAXBEL
  term.c_iflag |= IMAXBEL;
#endif /* IMAXBEL */
  //term.c_iflag |= ISTRIP; // not 8-bit clean
  term.c_oflag = OPOST | ONLCR;
  term.c_cflag = B9600 | CREAD;
  //term.c_cflag |=  PARENB | CS7; // not 8-bit
  term.c_cflag |= CS8; // 8-bit
  term.c_lflag = ISIG | IEXTEN | ICANON | ECHO | ECHOE | ECHOK;
#ifdef ECHOCTL
  term.c_lflag |= ECHOCTL;
#endif /* ECHOCTL */
#ifdef ECHOKE
  term.c_lflag |= ECHOKE;
#endif /* ECHOKE */
  term.c_cc[VINTR] = 003;		/* ^C */
  term.c_cc[VQUIT] = 034;		/* ^\ */
  term.c_cc[VERASE] = 0177;	/* DEL */
  term.c_cc[VKILL] = 025;		/* ^U */
  term.c_cc[VEOF] = 004;		/* ^D */
  term.c_cc[VSTART] = 021;	/* ^Q */
  term.c_cc[VSTOP] = 023;		/* ^S */
  term.c_cc[VSUSP] = 032;		/* ^Z */
#ifdef VREPRINT
  term.c_cc[VREPRINT] = 022;	/* ^R */
#endif /* VREPRINT */
#ifdef VWERASE
  term.c_cc[VWERASE] = 027;	/* ^W */
#endif /* VWERASE */
#ifdef VLNEXT
  term.c_cc[VLNEXT] = 026;	/* ^V */
#endif /* VLNEXT */
#ifdef VDSUSP
  term.c_cc[VDSUSP] = 031;	/* ^Y */
#endif /* VDSUSP */
#ifdef VDISCARD
  term.c_cc[VDISCARD] = 017;	/* ^O */
#endif /* VDISCARD */
  
  tcsetattr(ptyslave, TCSANOW, &term);

  pid = fork();
  if (!pid) {
    dup2(ptyslave, 0);
    dup2(ptyslave, 1);
    dup2(ptyslave, 2);
    setenv("TERM", "linux", 1);
#warning set WINDOWID, LINES, COLUMNS too
    execlp("/bin/sh", "/bin/sh", "-c", command, NULL);
  }

  printf("entering main loop\n");
  mainloop_console(ptymaster);

  close(ptymaster);
  close(ptyslave);

  printf("killing\n");
  kill_console();

  return 0;
}
